"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ember_cli_entities_1 = require("./lib/utils/ember-cli-entities");
const broccoli_debug_1 = __importDefault(require("broccoli-debug"));
const preprocessor_registry_1 = require("./plugins/preprocessor-registry");
const options_1 = require("./lib/options");
const ember_css_modules_1 = __importDefault(require("./plugins/ember-css-modules"));
const broccoli_file_creator_1 = require("broccoli-file-creator");
const broccoli_merge_trees_1 = __importDefault(require("broccoli-merge-trees"));
const addonPrototype = ember_cli_entities_1.addon({
    name: require(`${__dirname}/../package`).name,
    makeupOptions: undefined,
    parentAddon: undefined,
    get debugTree() {
        return broccoli_debug_1.default.buildDebugCallback(this.name);
    },
    included(includer) {
        this.computeOptions(includer);
        this._super.included.call(this, includer);
    },
    computeOptions(includer) {
        if (this.makeupOptions)
            return;
        this.makeupOptions = options_1.computeOptions(includer.options && includer.options.makeup);
        if (this.belongsToAddon()) {
            this.parentAddon = includer;
        }
    },
    belongsToAddon() {
        return Boolean(this.parent.parent);
    },
    /**
     * Integrate with other CSS processors, like ember-cli-sass.
     *
     * @see https://github.com/ember-cli/ember-cli-preprocess-registry#addon-usage
     */
    setupPreprocessorRegistry(type, registry) {
        preprocessor_registry_1.register(this, type, registry);
    },
    /**
     * Integrate with ember-css-modules.
     *
     * @see https://github.com/salsify/ember-css-modules/blob/master/docs/PLUGINS.md#plugins
     */
    createCssModulesPlugin(parent) {
        this.computeOptions(parent);
        return new ember_css_modules_1.default(parent, this);
    },
    treeForAddon(tree) {
        const originalTree = this.debugTree(this._super.treeForAddon.call(this, tree), 'treeForAddon:input');
        const configModuleName = `${this.name}/config`;
        const config = this.makeupOptions;
        const configFile = new broccoli_file_creator_1.Class(`${configModuleName}.js`, `define.exports('${configModuleName}', { default: ${JSON.stringify(config)} });`);
        const mergedTree = this.debugTree(new broccoli_merge_trees_1.default([originalTree, configFile], {
            annotation: 'ember-makeup:merge-config'
        }), 'treeForAddon:output');
        return mergedTree;
    }
});
exports.default = addonPrototype;
